<?php

/**
 * @file
 * dquarks module component handling.
 * Provides interface and database handling for editing components of a dquarks.
 *
 * @author IP-TECH
 */

/**
 * Overview form of all components for this dquarks.
 *
 * @param array $form_state
 *   An array of form values items.
 * @param object $node
 *   The node object
 *
 * @return Array
 *   Information about the component.
 */
function dquarks_components_form($form_state, $node) {
  $form = array(
    '#tree' => TRUE,
    '#node' => $node,
    'components' => array(),
  );

  $form['nid'] = array(
    '#type' => 'value',
    '#value' => $node->nid,
  );

  $options = array();
  foreach ($node->dquarks['components'] as $cid => $component) {
    $options[$cid] = check_plain($component['name']);
    $form['components'][$cid]['cid'] = array(
      '#type' => 'hidden',
      '#default_value' => $component['cid'],
    );
    $form['components'][$cid]['pid'] = array(
      '#type' => 'hidden',
      '#default_value' => $component['pid'],
    );
    $form['components'][$cid]['weight'] = array(
      '#type' => 'textfield',
      '#size' => 4,
      '#title' => t('Weight'),
      '#default_value' => $component['weight'],
    );
    $form['components'][$cid]['mandatory'] = array(
      '#type' => 'checkbox',
      '#title' => t('Mandatory'),
      '#default_value' => $component['mandatory'],
      '#access' => !in_array($component['type'], array('direction')),
    );
    $form['components'][$cid]['email'] = array(
      '#type' => 'checkbox',
      '#title' => t('E-mail'),
      '#default_value' => $component['email'],
      '#access' => !in_array($component['type'], array('direction')),
    );
  }

  $form['add']['name'] = array(
    '#type' => 'textfield',
    '#size' => 24,
  );

  $component_types = dquarks_load_components();
  natcasesort($component_types);
  $form['add']['type'] = array(
    '#type' => 'select',
    '#options' => $component_types,
    '#weight' => 3,
    '#default_value' => (isset($_GET['cid']) && isset($node->dquarks['components'][$_GET['cid']])) ? $node->dquarks['components'][$_GET['cid']]['type'] : 'textfield',
  );
  $form['add']['mandatory'] = array(
    '#type' => 'checkbox',
  );
  $form['add']['email'] = array(
    '#type' => 'checkbox',
    '#default_value' => 1,
  );
  $form['add']['cid'] = array(
    '#type' => 'hidden',
    '#default_value' => '',
  );
  $form['add']['pid'] = array(
    '#type' => 'hidden',
    '#default_value' => (isset($_GET['cid']) && isset($node->dquarks['components'][$_GET['cid']])) ? $node->dquarks['components'][$_GET['cid']]['pid'] : 0,
  );
  $form['add']['weight'] = array(
    '#type' => 'hidden',
    '#delta' => count($node->dquarks['components']) > 10 ? count($node->dquarks['components']) : 10,
    '#default_value' => (isset($_GET['cid']) && isset($node->dquarks['components'][$_GET['cid']])) ? $node->dquarks['components'][$_GET['cid']]['weight'] + 1 : count($node->dquarks['components']),
  );

  $form['add']['add'] = array(
    '#type' => 'submit',
    '#value' => t('Add'),
    '#weight' => 45,
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
    '#weight' => 45,
  );

  $form['publish'] = array(
    '#type' => 'submit',
    '#value' => t('Publish'),
    '#weight' => 50,
    '#access' => !$node->status,
  );
  $form['unpublish'] = array(
    '#type' => 'submit',
    '#value' => t('Unpublish'),
    '#weight' => 50,
    '#access' => $node->status,
  );

  return $form;
}

/**
 * Theme the node components form. Use a table to organize the components.
 *
 * @param array $form
 *   Contains the form array.
 *
 * @return string
 *   Formatted HTML form, ready for display.
 */
function theme_dquarks_components_form($form) {
  // Add CSS to display submission info. Don't preprocess because this CSS file
  // is used rarely.
  drupal_add_css(drupal_get_path('module', 'dquarks') . '/css/dquarks.css');
  drupal_add_js(drupal_get_path('module', 'dquarks') . '/js/dquarks.js');

  // Ce test permet d'eliminer l'erreur au niveau du premier composant.
  if ($form['components'][1]) {
    drupal_add_tabledrag('dquarks-components', 'order', 'sibling', 'dquarks-weight');
    drupal_add_tabledrag('dquarks-components', 'match', 'parent', 'dquarks-pid', 'dquarks-pid', 'dquarks-cid');
  }
  $node = $form['#node'];

  $headers = array(
    t('Name'),
    t('Type'),
    t('Value'),
    t('Mandatory'),
    t('E-mail'),
    t('Weight'),
    array('data' => t('Operations'), 'colspan' => 3),
  );
  $rows = array();
  $page1 = array(t('Page') . ' ' . 1 . ' :', '', '', '', '', '', '', '', '');
  $rows[] = array('data' => $page1, 'class' => 'tabledrag-leaf start_page');
  // Add a row containing form elements for a new item.
  unset($form['add']['name']['#title'], $form['add_type']['#description']);
  $form['add']['name']['#value'] = t('New component name');
  $form['add']['name']['#attributes']['class'] = 'dquarks-default-value';
  $form['add']['cid']['#attributes']['class'] = 'dquarks-cid';
  $form['add']['pid']['#attributes']['class'] = 'dquarks-pid';
  $form['add']['weight']['#attributes']['class'] = 'dquarks-weight';
  $row_data = array(
    array('data' => drupal_render($form['add']['name']), 'style' => "width:210px;"),
    drupal_render($form['add']['type']),
    '',
    drupal_render($form['add']['mandatory']),
    drupal_render($form['add']['email']),
    drupal_render($form['add']['cid']) . drupal_render($form['add']['pid']) . drupal_render($form['add']['weight']),
    array('colspan' => 3, 'data' => drupal_render($form['add']['add'])),
  );
  $add_form = array('data' => $row_data, 'class' => 'dquarks-add-form ');
  $form_rendered = FALSE;

  if (!empty($node->dquarks['components'])) {
    $add_form['class'] .= "draggable";
    $component_tree = array();
    $page_count = 1;
    _dquarks_components_tree_build($node->dquarks['components'], $component_tree, 0, $page_count);
    $component_tree = _dquarks_components_tree_sort($component_tree);

    /**
     * Build the table rows.
     *
     * @param object $node
     *   The node object.
     * @param int $cid
     *   The component id.
     * @param array $component
     *   The current component.
     * @param int $level
     *   The current level
     * @param array $form
     *   An array of form items.
     * @param array $rows
     *   An array of component of quiz.
     * @param array $add_form
     *   The form to add
     * @param int $num_page
     *   The number of page
     */
    function _dquarks_components_form_rows($node, $cid, $component, $level, &$form, &$rows, &$add_form, &$num_page = 1) {
      // Create presentable values.
      if (drupal_strlen($component['value']) > 30) {
        $component['value'] = drupal_substr($component['value'], 0, 30);
        $component['value'] .= '...';
      }
      $component['value'] = check_plain($component['value']);

      // Remove individual titles from the mandatory and weight fields.
      unset($form['components'][$cid]['mandatory']['#title']);
      unset($form['components'][$cid]['pid']['#title']);
      unset($form['components'][$cid]['weight']['#title']);
      unset($form['components'][$cid]['email']['#title']);

      // Add special classes for weight and parent fields.
      $form['components'][$cid]['cid']['#attributes']['class'] = 'dquarks-cid';
      $form['components'][$cid]['pid']['#attributes']['class'] = 'dquarks-pid';
      $form['components'][$cid]['weight']['#attributes']['class'] = 'dquarks-weight';

      // Build indentation for this row.
      $indents = '';
      for ($n = 1; $n <= $level; $n++) {
        $indents .= '<div class="indentation">&nbsp;</div>';
      }

      // Add each component to a table row.
      $row_data = array(
        $indents . filter_xss($component['name']),
        t($component['type']),
        ($component['value'] == '') ? '-' : $component['value'],
        drupal_render($form['components'][$cid]['mandatory']),
        drupal_render($form['components'][$cid]['email']),
        drupal_render($form['components'][$cid]['cid']) . drupal_render($form['components'][$cid]['pid']) . drupal_render($form['components'][$cid]['weight']),
        l(t('Edit'), 'node/' . $node->nid . '/edit/questions/' . $cid),
        l(t('Clone'), 'node/' . $node->nid . '/edit/questions/' . $cid . '/clone'),
        l(t('Delete'), 'node/' . $node->nid . '/edit/questions/' . $cid . '/delete'),
      );
      $row_class = 'draggable' . ($component['type'] != 'fieldset' ? ' tabledrag-leaf' : '');
      $row_class .= (($component['type'] == 'direction' || $component['type'] == 'pagebreak') ? ' end_page ' : '');
      $rows[] = array('data' => $row_data, 'class' => $row_class);
      if (isset($component['children']) && is_array($component['children'])) {
        foreach ($component['children'] as $cid => $component) {
          _dquarks_components_form_rows($node, $cid, $component, $level + 1, $form, $rows, $add_form, $num_page);
        }
      }

      // Add the add form if this was the last edited component.
      if (isset($_GET['cid']) && $component['cid'] == $_GET['cid'] && $add_form) {
        $add_form['data'][0]['data'] = $indents . $add_form['data'][0]['data'];
        $rows[] = $add_form;
        $add_form = FALSE;
      }
      if ($component['type'] == 'direction' || $component['type'] == 'pagebreak') {
        $num_page++;
        $saut_page = array(
          t('Page') . ' ' . $num_page . ' :',
          '', '', '', '', '', '', '', '',
        );
        $rows[] = array('data' => $saut_page, 'class' => '  start_page');
      }
    }

    $show_page_num = 1;
    foreach ($component_tree['children'] as $cid => $component) {
      _dquarks_components_form_rows($node, $cid, $component, 0, $form, $rows, $add_form, $show_page_num);
    }
  }
  else {
    $rows[] = array(array('data' => t('No Components, add a component below.'), 'colspan' => 9));
  }

  // Append the add form if not already printed.
  if ($add_form) {
    $rows[] = $add_form;
  }

  $output = '';
  $output .= theme('table', $headers, $rows, array('id' => 'dquarks-components', "style" => "width:100%;"));
  $output .= drupal_render($form);
  return $output;
}

/**
 * Implements hook_validate().
 *
 * @param array $form
 *   An array of form items.
 * @param array $form_state
 *   An array of form values items.
 */
function dquarks_components_form_validate($form, &$form_state) {
  if (isset($_POST['op']) && $_POST['op'] == t('Add') && drupal_strlen(trim($form_state['values']['add']['name'])) <= 0) {
    form_set_error('add][name', t('When adding a new question, the name field is required.'));
  }
}

/**
 * Implements hook_submit().
 *
 * @param array $form
 *   An array of form items.
 * @param array $form_state
 *   An array of form values items.
 *
 * @return int
 *   The id of node.
 */
function dquarks_components_form_submit($form, &$form_state) {
  $node = node_load($form_state['values']['nid']);

  // Update all mandatory and weight values.
  foreach ($node->dquarks['components'] as $cid => $component) {
    if ($component['pid'] != $form_state['values']['components'][$cid]['pid'] || $component['weight'] != $form_state['values']['components'][$cid]['weight'] || $component['mandatory'] != $form_state['values']['components'][$cid]['mandatory'] || $component['email'] != $form_state['values']['components'][$cid]['email']) {
      $component['weight'] = $form_state['values']['components'][$cid]['weight'];
      $component['mandatory'] = $form_state['values']['components'][$cid]['mandatory'];
      $component['email'] = $form_state['values']['components'][$cid]['email'];
      $component['pid'] = $form_state['values']['components'][$cid]['pid'];
      $component['nid'] = $node->nid;
      dquarks_component_update($component);
    }
  }

  if (isset($_POST['op']) && $_POST['op'] == t('Publish')) {
    $node->status = 1;
    node_save($node);
    drupal_set_message(t('Your quiz has been published.'));
    return 'node/' . $node->nid;
  }
  if (isset($_POST['op']) && $_POST['op'] == t('Unpublish')) {
    $node->status = 0;
    node_save($node);
    drupal_set_message(t("This quiz is currently unpublished. After finishing your changes to the quiz, use the <em>Publish</em> button below."));
    return 'node/' . $node->nid;
  }
  elseif (isset($_POST['op']) && $_POST['op'] == t('Add')) {
    $component = $form_state['values']['add'];
    $form_state['redirect'] = array('node/' . $node->nid . '/edit/questions/new/' . $component['type'], 'name=' . urlencode($component['name']) . '&mandatory=' . $component['mandatory'] . '&email=' . $component['email'] . '&pid=' . $component['pid'] . '&weight=' . $component['weight']);
  }
  else {
    drupal_set_message(t('The component positions and mandatory values have been updated.'));
    // Since dquarks components have been updated but the node itself has not
    // been saved, it is necessary to explicitly clear the cache to make sure
    // the updated dquarks is visible to anonymous users.
    cache_clear_all();
  }
}

/**
 *  Edit component.
 *
 * @param array $form_state
 *   An array of form values items.
 * @param object $node
 *   The node id.
 * @param array $component
 *   The component of quiz.
 * @param boolean $clone
 *   True, if a copy of component.
 *
 * @return array
 *   The details of component.
 */
function dquarks_component_edit_form(&$form_state, $node, $component, $clone = FALSE) {
  drupal_set_title(t('Edit component: @name (@type)', array('@name' => $component['name'], '@type' => t($component['type']))));

  // Print the correct field type specification.
  // We always need: name and description.
  $form = array(
    '#tree' => TRUE,
  );
  $form['type'] = array(
    '#type' => 'value',
    '#value' => $component['type'],
  );
  $form['nid'] = array(
    '#type' => 'value',
    '#value' => $node->nid,
  );
  $form['cid'] = array(
    '#type' => 'value',
    '#value' => isset($component['cid']) ? $component['cid'] : NULL,
  );
  $form['clone'] = array(
    '#type' => 'value',
    '#value' => $clone,
  );
  $form['name'] = array(
    '#type' => 'textfield',
    '#default_value' => $component['name'],
    '#title' => t('Label'),
    '#description' => t('This is used as a descriptive label when displaying this quiz element.'),
    '#required' => TRUE,
    '#weight' => -2,
    '#maxlength' => 255,
  );
  $form['extra']['description'] = array(
    '#type' => 'textarea',
    '#default_value' => isset($component['extra']['description']) ? $component['extra']['description'] : '',
    '#title' => t('Description'),
    '#description' => t('A short description of the field used as help for the user when he/she uses the quiz.') . theme('dquarks_token_help'),
    '#weight' => -1,
  );

  $form['advanced'] = array(
    '#type' => 'fieldset',
    '#title' => t('Advanced settings'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#tree' => FALSE,
    'weight' => 20,
  );
  $form['advanced']['form_key'] = array(
    '#type' => 'textfield',
    '#default_value' => empty($component['form_key']) ? _dquarks_safe_name($component['name']) : $component['form_key'],
    '#title' => t('Field Key'),
    '#description' => t('Enter a machine readable key for this quiz element. May contain only lowercase alphanumeric characters and underscores. This key will be used as the name attribute of the quiz element. This value has no effect on the way data is saved, but may be helpful if using Additional Processing or Validation code.'),
    '#required' => TRUE,
    '#weight' => -1,
  );
  $form['advanced']['mandatory'] = array(
    '#type' => 'checkbox',
    '#title' => t('Mandatory'),
    '#default_value' => ($component['mandatory'] == '1' ? TRUE : FALSE),
    '#description' => t('Check this option if the user must enter a value.'),
    '#weight' => 2,
    '#access' => !in_array($component['type'], array('direction')),
  );
  $form['advanced']['email'] = array(
    '#type' => 'checkbox',
    '#title' => t('Include in e-mails'),
    '#default_value' => ($component['email'] == '1' ? TRUE : FALSE),
    '#description' => t('If checked, submitted values from this component will be included in e-mails.'),
    '#weight' => 2,
    '#access' => !in_array($component['type'], array('direction')),
  );

  if (variable_get('dquarks_enable_fieldset', TRUE) && is_array($node->dquarks['components'])) {
    $options = array('0' => t('Root'));
    foreach ($node->dquarks['components'] as $existing_cid => $value) {
      if ($value['type'] == 'fieldset' && (!isset($component['cid']) || $existing_cid != $component['cid'])) {
        $options[$existing_cid] = $value['name'];
      }
    }
    $form['advanced']['pid'] = array(
      '#type' => 'select',
      '#title' => t('Parent Fieldset'),
      '#default_value' => $component['pid'],
      '#description' => t('Optional. You may organize your quiz by placing this component inside another fieldset.'),
      '#options' => $options,
      '#weight' => 3,
    );
  }

  $form['advanced']['weight'] = array(
    '#type' => 'textfield',
    '#size' => 4,
    '#title' => t('Weight'),
    '#default_value' => $component['weight'],
    '#description' => t('Optional. In the menu, the heavier items will sink and the lighter items will be positioned nearer the top.'),
    '#weight' => 4,
  );

  // Add the fields specific to this component type:
  // Load all component types.
  dquarks_load_components();
  $edit_function = '_dquarks_edit_' . $component['type'];
  $additional_form_elements = array();
  if (function_exists($edit_function)) {
    // Call the component render function.
    $additional_form_elements = $edit_function($component, $node, $form_state);
  }
  else {
    drupal_set_message(t('The quiz component of type @type does not have an edit function defined.', array('@type' => $component['type'])));
  }

  // Merge the additional fields with the current fields:
  if (isset($additional_form_elements['extra'])) {
    $extra_fields_copy = $form['extra'];
    $form['extra'] = array_merge($extra_fields_copy, $additional_form_elements['extra']);
    unset($additional_form_elements['extra']);
  }
  if (isset($additional_form_elements['advanced'])) {
    $advanced_fields_copy = $form['advanced'];
    $form['advanced'] = array_merge($advanced_fields_copy, $additional_form_elements['advanced']);
    unset($additional_form_elements['advanced']);
  }
  $form = array_merge($form, $additional_form_elements);

  // Add the submit button.
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
    '#weight' => 5,
  );
  $form['apply'] = array(
    '#type' => 'submit',
    '#value' => t('Apply'),
    '#weight' => 6,
  );
  $form['cancel'] = array(
    '#type' => 'submit',
    '#value' => t('Cancel'),
    '#weight' => 7,
  );
  return $form;
}

/**
 * Field name validation for the dquarks unique key. Must be alphanumeric.
 */
function dquarks_component_edit_form_validate($form, &$form_state) {
  $node = node_load($form_state['values']['nid']);

  if (!preg_match('!^[a-z0-9_]+$!', $form_state['values']['form_key'])) {
    form_set_error('form_key', t('The field key %field_key is invalid. Please include only lowercase alphanumeric characters and underscores.', array('%field_key' => $form_state['values']['form_key'])));
  }

  foreach ($node->dquarks['components'] as $cid => $component) {
    if (($component['cid'] != $form_state['values']['cid'] || $form_state['values']['clone']) && ($component['pid'] == $form_state['values']['pid']) && (strcasecmp($component['form_key'], $form_state['values']['form_key']) == 0)) {
      form_set_error('form_key', t('The field key %field_key is already in use by the field labeled %existing_field. Please use a unique key.', array('%field_key' => $form_state['values']['form_key'], '%existing_field' => $component['name'])));
    }
  }

  // Let the field do any additional validation.
  dquarks_load_components($form_state['values']['type']);
  $validate_function = '_dquarks_edit_validate_' . $form_state['values']['type'];
  if (function_exists($validate_function)) {
    $validate_function($form, $form_state);
  }
}

/**
 * Implements hook_submit().
 *
 * @param array $form
 *   An array of form items.
 * @param array $form_state
 *   An array of form values items.
 */
function dquarks_component_edit_form_submit($form, &$form_state) {
  $node = node_load($form_state['values']['nid']);
  if (isset($_POST['op']) && $_POST['op'] == t('Add')) {
    $rule = $form_state['values']['add'];
    $form_state['redirect'] = array(
      'node/' . $node->nid . '/edit/questions/' . $rule['cid'] . '/addrule',
      'test_name=' . urlencode($rule['test_name']) . '&test_type=' . $rule['test_type'] . '&tested_field=' . $rule['tested_field'] . '&testvalue=' . urlencode($rule['testvalue']) . '&next_jump=' . $rule['next_jump'] . '&weight=' . $rule['weight']);
  }
  elseif (isset($_POST['op']) && $_POST['op'] == t('Cancel')) {
    if (isset($form_state['cid']['#value'])) {
      $cid = $form_state['cid']['#value'];
      $form_state['redirect'] = array('node/' . $node->nid . '/edit/questions/' . $cid);
    }
    else {
      $form_state['redirect'] = array('node/' . $node->nid . '/edit/questions/');
    }
  }
  else {
    // Remove empty extra values.
    if (isset($form_state['values']['extra'])) {
      foreach ($form_state['values']['extra'] as $key => $value) {
        if ($value === '') {
          unset($form_state['values']['extra'][$key]);
        }
      }
    }
    dquarks_load_components($form_state['values']['type']);
    $validate_function = '_dquarks_edit_submit_' . $form_state['values']['type'];
    if (function_exists($validate_function)) {
      $validate_function($form, $form_state);
    }
    // Remove empty attribute values.
    if (isset($form_state['values']['extra']['attributes'])) {
      foreach ($form_state['values']['extra']['attributes'] as $key => $value) {
        if ($value === '') {
          unset($form_state['values']['extra']['attributes'][$key]);
        }
      }
    }
    $cid = $form['cid']['#value'];
    if ($form_state['values']['clone']) {
      drupal_set_message(t('Component %name cloned.', array('%name' => $form_state['values']['name'])));
      dquarks_component_clone($node, $form_state['values']);
    }
    elseif (!empty($form_state['values']['cid'])) {
      drupal_set_message(t('Component %name updated.', array('%name' => $form_state['values']['name'])));
      dquarks_component_update($form_state['values']);
    }
    else {
      drupal_set_message(t('New component %name added.', array('%name' => $form_state['values']['name'])));
      $cid = dquarks_component_insert($form_state['values']);
    }
    // Since dquarks components have been updated but the node itself has not
    // been saved, it is necessary to explicitly clear the cache to make sure
    // the updated dquarks is visible to anonymous users.
    cache_clear_all();
    if (isset($_POST['op']) && $_POST['op'] == t('Apply')) {
      $form_state['redirect'] = array('node/' . $form_state['values']['nid'] . '/edit/questions/' . $cid);
    }
    else {
      $form_state['redirect'] = array('node/' . $form_state['values']['nid'] . '/edit/questions', isset($cid) ? 'cid=' . $cid : '');
    }
  }
}

/**
 * AHAH callback function when adding new element to a component.
 *
 * @param object $node
 *   The node object
 * @param int $id
 *   The id of question to delete.
 */
function dquarks_component_edit_callback($node, $id = NULL) {
  $form_state = array('storage' => NULL, 'submitted' => FALSE);
  $form_build_id = check_plain($_POST['form_build_id']);
  $form = form_get_cache($form_build_id, $form_state);
  $args = $form['#parameters'];
  $form_id = array_shift($args);
  $_POST["del_question"] = $id;
  $form_state['post'] = $form['#post'] = $_POST;
  $form['#programmed'] = $form['#redirect'] = FALSE;

  dquarks_load_components();
  drupal_process_form($form_id, $form, $form_state);
  $form = drupal_rebuild_form($form_id, $form_state, $args, $form_build_id);
  $update = $form_state["clicked_button"]["#ahah"]["update_component"];
  if (function_exists($update)) {
    $output = drupal_render($update($form));
  }
  else {
    $output = drupal_render($form);
  }
  $js_settings = drupal_add_js(NULL, NULL, 'header');
  print drupal_json(array(
    'status' => TRUE,
    'data' => $output,
    'settings' => call_user_func_array('array_merge_recursive', $js_settings['setting']),
  ));
  exit();
}

/**
 * Return a confirmation message to delete submission.
 *
 * @param array $form_state
 *   An array of form values items.
 * @param object $node
 *   The node id.
 * @param array $component
 *   The current component.
 *
 * @return string
 *   Message of confirmation
 */
function dquarks_component_delete_form($form_state, $node, $component) {
  $cid = $component['cid'];

  $form = array();
  $form['node'] = array(
    '#type' => 'value',
    '#value' => $node,
  );
  $form['component'] = array(
    '#type' => 'value',
    '#value' => $component,
  );

  if ($node->dquarks['components'][$cid]['type'] == 'fieldset') {
    $question = t('Delete the %name fieldset?', array('%name' => $node->dquarks['components'][$cid]['name']));
    $description = t('This will immediately delete the %name fieldset and all children elements within %name from the %dquarks quiz. This cannot be undone.', array('%name' => $node->dquarks['components'][$cid]['name'], '%dquarks' => $node->title));
  }
  else {
    $question = t('Delete the %name component?', array('%name' => $node->dquarks['components'][$cid]['name']));
    $description = t('This will immediately delete the %name component from the %dquarks quiz. This cannot be undone.', array('%name' => $node->dquarks['components'][$cid]['name'], '%dquarks' => $node->title));
  }

  return confirm_form($form, $question, 'node/' . $node->nid . '/edit/questions', $description, t('Delete'));
}

/**
 * Implements hook_submit().
 *
 * @param array $form
 *   An array of form items.
 * @param array $form_state
 *   An array of form values items.
 */
function dquarks_component_delete_form_submit($form, &$form_state) {
  drupal_set_message(t('Component %name deleted.', array('%name' => $form_state['values']['component']['name'])));
  dquarks_component_delete($form_state['values']['node'], $form_state['values']['component']);
  $form_state['redirect'] = 'node/' . $form_state['values']['node']->nid . '/edit/questions';
}

/**
 * Insert a new component into the database.
 *
 * @param array $component
 *   A full component containing fields from the component form.
 */
function dquarks_component_insert(&$component) {
  db_lock_table('dquarks_component');
  $component['cid'] = isset($component['cid']) ? $component['cid'] : db_result(db_query('SELECT MAX(cid) FROM {dquarks_component} WHERE nid = %d', $component['nid'])) + 1;
  $component['value'] = isset($component['value']) ? $component['value'] : NULL;
  $component['mandatory'] = isset($component['mandatory']) ? $component['mandatory'] : 0;
  db_query("INSERT INTO {dquarks_component} (nid, cid, pid, form_key, name, type, value, extra, mandatory, weight, email) VALUES (%d, %d, %d, '%s', '%s', '%s', '%s', '%s', %d, %d, %d)", $component['nid'], $component['cid'], $component['pid'], $component['form_key'], $component['name'], $component['type'], $component['value'], serialize($component['extra']), $component['mandatory'], $component['weight'], $component['email']);
  db_unlock_tables('dquarks_component');
  $insert_function = '_dquarks_after_insert_' . $component['type'];
  if (function_exists($insert_function)) {
    $insert_function($component);
  }
  db_unlock_tables();
  return $component['cid'];
}

/**
 * Update an existing component with new values.
 *
 * @param array $component
 *   A full component containing a nid, cid, and all other fields from the
 *   component form. Additional properties are stored in the extra array.
 */
function dquarks_component_update($component) {
  $component['value'] = isset($component['value']) ? $component['value'] : NULL;
  $component['mandatory'] = isset($component['mandatory']) ? $component['mandatory'] : 0;
  $return = db_query("UPDATE {dquarks_component} SET pid = %d, form_key = '%s', name = '%s', type = '%s', value = '%s', extra = '%s', mandatory = %d, weight = %d, email = %d WHERE nid = %d AND cid = %d", $component['pid'], $component['form_key'], $component['name'], $component['type'], $component['value'], serialize($component['extra']), $component['mandatory'], $component['weight'], $component['email'], $component['nid'], $component['cid']);
  $update_function = '_dquarks_after_update_' . $component['type'];
  if (function_exists($update_function)) {
    $update_function($component);
  }
  return $return;
}

/**
 * Load all submissions and allow the component to delete each one.
 *
 * @param object $node
 *   The node id.
 * @param array $component
 *   The component to delete.
 */
function dquarks_component_delete($node, $component) {
  // Check if a delete function is available for this component. If so,
  // load all submissions and allow the component to delete each one.
  $delete_function = '_dquarks_delete_' . $component['type'];
  if (function_exists($delete_function)) {
    module_load_include('inc', 'dquarks', 'includes/dquarks_submissions');
    $submissions = dquarks_get_submissions($node->nid);
    foreach ($submissions as $submission) {
      if (isset($submission->data[$component['cid']])) {
        $delete_function($submission->data[$component['cid']], $component);
      }
    }
  }
  $after_delete_function = '_dquarks_after_delete_' . $component['type'];
  if (function_exists($after_delete_function)) {
    $after_delete_function($component);
  }


  // Remove database entries.
  db_query('DELETE FROM {dquarks_component} WHERE nid = %d AND cid = %d', $node->nid, $component['cid']);
  db_query('DELETE FROM {dquarks_submitted_data} WHERE nid = %d AND cid = %d', $node->nid, $component['cid']);

  // Delete all elements under this element.
  $result = db_query('SELECT cid FROM {dquarks_component} WHERE nid = %d AND pid = %d', $node->nid, $component['cid']);
  while ($row = db_fetch_object($result)) {
    $component = $node->dquarks['components'][$row->cid];
    dquarks_component_delete($node, $component);
  }
}

/**
 * Recursively insert components into the database.
 *
 * @param object $node
 *   The node object containing the current dquarks.
 * @param array $component
 *   A full component containing fields from the component form.
 */
function dquarks_component_clone(&$node, &$component) {
  $original_cid = $component['cid'];
  unset($component['cid']);
  $new_cid = dquarks_component_insert($component);
  if ($component['type'] == 'fieldset') {
    foreach ($node->dquarks['components'] as $cid => $child_component) {
      if ($child_component['pid'] == $original_cid) {
        $child_component['pid'] = $new_cid;
        dquarks_component_clone($node, $child_component);
      }
    }
  }
  return $new_cid;
}

/**
 * Populate a component with the defaults for that type.
 */
function dquarks_component_defaults(&$component) {
  dquarks_load_components();

  $function = '_dquarks_defaults_' . $component['type'];
  if (function_exists($function)) {
    $defaults = $function();
    foreach ($defaults as $key => $default) {
      if (!isset($component[$key])) {
        $component[$key] = $default;
      }
    }
    foreach ($defaults['extra'] as $extra => $default) {
      if (!isset($component['extra'][$extra])) {
        $component['extra'][$extra] = $default;
      }
    }
  }
}

/**
 * Define rules for a component.
 *
 * @param array $form_state
 *   The current form values.
 * @param object $node
 *   The node id.
 * @param array $component
 *   The current component.
 *
 * @return Array
 *   Call to a function or display message
 */
function dquarks_component_addrule_form(&$form_state, $node, $component) {
  // Load all component types.
  dquarks_load_components();
  $addrule_function = '_dquarks_addrule_' . $component['type'];
  if (function_exists($addrule_function)) {
    // Call the component render function.
    return $addrule_function($form_state, $node, $component);
  }
  else {
    drupal_set_message(t('The quiz component of type @type does not have an addrule function defined.', array('@type' => $component['type'])));
  }
}

/**
 * Call to the function of delete rule of direction component type.
 *
 * @param array $form_state
 *   An array of form items.
 * @param object $node
 *   The dquarks node object.
 * @param array $component
 *   The component of the dquarks node.
 * @param string $rule
 *   The rule to delete.
 *
 * @return Array
 *   Call another function or display message
 */
function dquarks_component_delrule_form($form_state, $node, $component, $rule) {
  // Load all component types.
  dquarks_load_components();
  $delrule_function = '_dquarks_delrule_' . $component['type'];
  if (function_exists($delrule_function)) {
    // Call the component render function.
    return $delrule_function($form_state, $node, $component, $rule);
  }
  else {
    drupal_set_message(t('The quiz component of type @type does not have an delrule function defined.', array('@type' => $component['type'])));
  }
}

/**
 * Delete rule of direction component.
 *
 * @param array $form
 *   An array of form items.
 * @param array $form_state
 *   An array of form items.
 */
function dquarks_component_delrule_form_submit($form, &$form_state) {
  drupal_set_message(t('The %name role deleted.', array('%name' => $form_state['values']['role']['test_name'])));
  // Load all component types.
  dquarks_load_components();
  $delrule_function = '_dquarks_delrule_submit_' . $form_state['values']['component']['type'];
  if (function_exists($delrule_function)) {
    // Call the component render function.
    $delrule_function($form, $form_state);
  }
  else {
    drupal_set_message(t('The quiz component of type @type does not have an delrule function defined.', array('@type' => $form_state['values']['component']['type'])));
  }
  $form_state['redirect'] = 'node/' . $form_state['values']['node']->nid . '/edit/questions/' . $form_state['values']['component']['cid'];
}

/**
 * Load a specific rules of direction component.
 *
 * @param object $node
 *   The dquarks node object.
 * @param object $component
 *   The direction component of quizz.
 * @param int $ruleid
 *   Id of rule.
 *
 * @return Array
 *   The rule of direction component
 */
function dquarks_component_loadrole($node, $component, $ruleid) {
  // Load all component types.
  dquarks_load_components();
  $loadrole_function = '_dquarks_loadrole_' . $component['type'];
  if (function_exists($loadrole_function)) {
    // Call the component render function.
    return $loadrole_function($node, $component, $ruleid);
  }
  else {
    return NULL;
  }
}
